==> pathena-0.3.3.tgz: .pathena/lib/db_operations.py <==
# Database client interface operations.  Also used by indexing functions.

from db_connect        import *
from file_type_attribs import *

_temp_file = new_temp_file_name()


#-----------------

def getIndexedModTime(path):
    db_attribs = do_query("select mod_time from file_descriptors "
                          "where abs_path = %s", path)
    if db_attribs: return db_attribs[0][0]
    else:          return 0

# Retrieve latest stats on GiST vectors.  vector = 'path' or 'content'.
# Choose most or least frequent words.
# Return ((total, singletons, avg(ndoc), avg(nentry)), <most/least freq>).

def getTsearchStats(vector, limit, most_freq=1):
    try:
        counts = do_query("select avg(ndoc), avg(nentry) from latest_%s_stats"
                          % vector)
    except DatabaseError:
        return (0, 0, 0, 0), []
    avg_ndoc   = counts[0][0] or 0    # counts could be None
    avg_nentry = counts[0][1] or 0
    stats = ((do_query("select count(*) from latest_%s_stats" % vector
                       )[0][0],
              do_query("select count(*) from latest_%s_stats "
                       "where nentry = 1" % vector)[0][0],
              avg_ndoc, avg_nentry))
    if most_freq:
        histo_query = "select * from latest_%s_stats limit %%s"
    else:
        histo_query = \
            "select * from latest_%s_stats order by nentry asc limit %%s"
    return stats, do_query(histo_query % vector, str(limit))

def update_summary_stats():
    total   = do_query("SELECT count(*) FROM file_descriptors")[0][0]
    content = do_query("SELECT count(*) FROM file_descriptors "
                       "WHERE content_vector IS NOT NULL")[0][0]
    cached  = do_query("SELECT count(*) FROM file_descriptors "
                       "WHERE text_content IS NOT NULL")[0][0]
    do_query_effect("UPDATE summary_stats "
                    "SET num_paths=%s, num_indexed=%s, num_cached=%s",
                    *map(str, (total, content, cached)))

def refreshTsearchStats():
    # truncating tables doesn't work; seems to be locking behavior
    do_query_effect("delete from latest_path_stats")
    do_query_effect("delete from latest_content_stats")
    do_query_effect("select update_path_stats()")
    do_query_effect("select update_content_stats()")

def vacuumDatabase(delay=1):
    # vacuum, analyze using PG-provided script:
    log_file = os.path.join(log_dir, db_ops_log_name)
    msg_cmd = 'echo -e "\nStarting PostgreSQL vacuum process on $(date)."'
    vacuum_cmd = os.path.expandvars('$PGBIN/vacuumdb -p $PGPORT -d pathena -z')
    os.system('(sleep %s; %s; %s) >> %s 2>&1 &' %
              (delay, msg_cmd, vacuum_cmd, log_file))


#-----------------

# Retrieve a file's text content either by direct conversion or by
# check if a file's content has been cached in the database, then by
# conversion.  If any problems, content is empty string.
# Return cached state, title, metadata and content.
# Cached state:
#   0 - not cacheable
#   1 - cacheable and cached content was found
#   2 - cacheable but no cached content is available

def get_cached_content_plus(type_class, path, usage=0):
    cacheable = type_class.cache_conversion(path)
    if not cacheable:
        return decompress_convert(type_class, path, usage)
    db_attribs = do_query("select file_title, file_metadata, text_content "
                          "from file_descriptors where abs_path = %s", path)
    if db_attribs:
        title, metadata, content = db_attribs[0]
    else:
        # file not found in database
        return 2, '', '', ''
    if not content:
        # content was not cached
        return 2, '', '', ''
    return 1, title, metadata, \
           type_class.convert_cached_content(content, usage)

# Decompress file if necessary, then return title, metadata and content.
# Return flag 0 to indicate data was not cached.

def decompress_convert(type_class, path, usage=0):
    base, ext = os.path.splitext(path)
    if ext[1:] in compress_ext_command and \
       os.path.splitext(base)[1][1:] in file_ext_class:
        decompress_cmd = compress_ext_command[ext[1:]]
        make_shell_command('%s > %s' % (decompress_cmd, _temp_file))(path)
        title    = type_class.extract_title(_temp_file, base)
        metadata = type_class.extract_metadata(_temp_file, base)
        content  = type_class.convert_file(_temp_file, usage)
        try:    os.remove(_temp_file)
        except: pass
        return 0, title, metadata, content
    else:
        return 0, type_class.extract_title(path), \
               type_class.extract_metadata(path), \
               type_class.convert_file(path, usage)


# Ask Tsearch2 to compute a headline for the designated query by
# extracting text from the file.  Truncate output to LIMIT words.

def get_headline(type_class, path, terms, limit, specials='', mode=0):
    if vacuous_content_query(terms): return ''
    headline = do_query("select get_headline(%s, %s, %s, %s, %s, %s)",
                        type_class.file_type_name(), path,
                        '\n'.join(terms), str(limit), specials, str(mode))
    return headline[0][0]

==> pathena-0.3.4.tgz: .pathena/lib/db_operations.py <==
# Database client interface operations.  Also used by indexing functions.

from db_connect        import *
from file_type_attribs import *

_temp_file = new_temp_file_name()


#-----------------

def getIndexedModTime(path):
    db_attribs = do_query("select mod_time from file_descriptors "
                          "where abs_path = %s", path)
    if db_attribs: return db_attribs[0][0]
    else:          return 0

# Retrieve latest stats on GiST vectors.  vector = 'path' or 'content'.
# Choose most or least frequent words.
# Return ((total, singletons, avg(ndoc), avg(nentry)), <most/least freq>).

def getTsearchStats(vector, limit, most_freq=1):
    try:
        counts = do_query("select avg(ndoc), avg(nentry) from latest_%s_stats"
                          % vector)
    except DatabaseError:
        return (0, 0, 0, 0), []
    avg_ndoc   = counts[0][0] or 0    # counts could be None
    avg_nentry = counts[0][1] or 0
    stats = ((do_query("select count(*) from latest_%s_stats" % vector
                       )[0][0],
              do_query("select count(*) from latest_%s_stats "
                       "where nentry = 1" % vector)[0][0],
              avg_ndoc, avg_nentry))
    if most_freq:
        histo_query = "select * from latest_%s_stats limit %%s"
    else:
        histo_query = \
            "select * from latest_%s_stats order by nentry asc limit %%s"
    return stats, do_query(histo_query % vector, str(limit))

def update_summary_stats():
    total   = do_query("SELECT count(*) FROM file_descriptors")[0][0]
    content = do_query("SELECT count(*) FROM file_descriptors "
                       "WHERE content_vector IS NOT NULL")[0][0]
    cached  = do_query("SELECT count(*) FROM file_descriptors "
                       "WHERE text_content IS NOT NULL")[0][0]
    do_query_effect("UPDATE summary_stats "
                    "SET num_paths=%s, num_indexed=%s, num_cached=%s",
                    *map(str, (total, content, cached)))

def refreshTsearchStats():
    # truncating tables doesn't work; seems to be locking behavior
    do_query_effect("delete from latest_path_stats")
    do_query_effect("delete from latest_content_stats")
    do_query_effect("select update_path_stats()")
    do_query_effect("select update_content_stats()")

def vacuumDatabase(delay=1):
    # vacuum, analyze using PG-provided script:
    log_file = os.path.join(log_dir, db_ops_log_name)
    msg_cmd = 'echo -e "\nStarting PostgreSQL vacuum process on $(date)."'
    vacuum_cmd = os.path.expandvars('$PGBIN/vacuumdb -p $PGPORT -d pathena -z')
    os.system('(sleep %s; %s; %s) >> %s 2>&1 &' %
              (delay, msg_cmd, vacuum_cmd, log_file))


#-----------------

# Retrieve a file's text content either by direct conversion or by
# check if a file's content has been cached in the database, then by
# conversion.  If any problems, content is empty string.
# Return cached state, title, metadata and content.
# Cached state:
#   0 - not cacheable
#   1 - cacheable and cached content was found
#   2 - cacheable but no cached content is available

def get_cached_content_plus(type_class, path, usage=0):
    cacheable = type_class.cache_conversion(path)
    if not cacheable:
        return decompress_convert(type_class, path, usage)
    db_attribs = do_query("select file_title, file_metadata, text_content "
                          "from file_descriptors where abs_path = %s", path)
    if db_attribs:
        title, metadata, content = db_attribs[0]
    else:
        # file not found in database
        return 2, '', '', ''
    if not content:
        # content was not cached
        return 2, '', '', ''
    return 1, title, metadata, \
           type_class.convert_cached_content(content, usage)

# Decompress file if necessary, then return title, metadata and content.
# Return flag 0 to indicate data was not cached.

def decompress_convert(type_class, path, usage=0):
    base, ext = os.path.splitext(path)
    ext = ext[1:].lower()
    if ext in compress_ext_command and \
       os.path.splitext(base)[1][1:].lower() in file_ext_class:
        decompress_cmd = compress_ext_command[ext]
        make_shell_command('%s > %s' % (decompress_cmd, _temp_file))(path)
        title    = type_class.extract_title(_temp_file, base)
        metadata = type_class.extract_metadata(_temp_file, base)
        content  = type_class.convert_file(_temp_file, usage)
        try:    os.remove(_temp_file)
        except: pass
        return 0, title, metadata, content
    else:
        return 0, type_class.extract_title(path), \
               type_class.extract_metadata(path), \
               type_class.convert_file(path, usage)


# Ask Tsearch2 to compute a headline for the designated query by
# extracting text from the file.  Truncate output to LIMIT words.

def get_headline(type_class, path, terms, limit, specials='', mode=0):
    if vacuous_content_query(terms): return ''
    headline = do_query("select get_headline(%s, %s, %s, %s, %s, %s)",
                        type_class.file_type_name(), path,
                        '\n'.join(terms), str(limit), specials, str(mode))
    return headline[0][0]
